@@ -2,7 +2,7 @@
  * vlc_input.h: Core input structures
  *****************************************************************************
  * Copyright (C) 1999-2015 VLC authors and VideoLAN
- * $Id: 76ecee52675858ab7b63f50bb1fb2d5760009663 $
+ * $Id: 6faae4638f3e94d58acbfaa3432487c007acaf7f $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -360,8 +360,6 @@ typedef enum input_event_type_e
     INPUT_EVENT_ITEM_META,
     /* input_item_t info has changed */
     INPUT_EVENT_ITEM_INFO,
-    /* input_item_t name has changed */
-    INPUT_EVENT_ITEM_NAME,
     /* input_item_t epg has changed */
     INPUT_EVENT_ITEM_EPG,
 
@@ -438,9 +436,7 @@ enum input_query_e
     INPUT_ADD_INFO,   /* arg1= char* arg2= char* arg3=...     res=can fail */
     INPUT_REPLACE_INFOS,/* arg1= info_category_t *            res=cannot fail */
     INPUT_MERGE_INFOS,/* arg1= info_category_t *              res=cannot fail */
-    INPUT_GET_INFO,   /* arg1= char* arg2= char* arg3= char** res=can fail */
     INPUT_DEL_INFO,   /* arg1= char* arg2= char*              res=can fail */
-    INPUT_SET_NAME,   /* arg1= char* res=can fail    */
 
     /* bookmarks */
     INPUT_GET_BOOKMARK,    /* arg1= seekpoint_t *               res=can fail */
