@@ -2,7 +2,7 @@
  * vlc_services_discovery.h : Services Discover functions
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: 78aeed7c2084da8da73b3a56dd21b44c9e593288 $
+ * $Id: 6f3dd777398599169dce003260b1018917822871 $
  *
  * Authors: Pierre d'Herbemont <pdherbemont # videolan.org>
  *
@@ -195,7 +195,7 @@ static inline void services_discovery_AddItemCat(services_discovery_t *sd,
                                                  input_item_t *item,
                                                  const char *category)
 {
-    return sd->owner.item_added(sd, item, NULL, category);
+    return sd->owner.item_added(sd, NULL, item, category);
 }
 
 /**
