@@ -2,7 +2,7 @@
  * utils.c: helper functions
  *****************************************************************************
  * Copyright (C) 2010 VLC authors and VideoLAN
- * $Id: ab62f0bacf850e20f1959335fb1427fa5951db0e $
+ * $Id: a7d7ee1abf74540c08bc7391d5abae9d92e5d5bb $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -223,7 +223,8 @@ void CopyOmxPicture( int i_color_format, picture_t *p_pic,
         copy_cache_t *p_surface_cache = (copy_cache_t*)p_architecture_specific->data;
         uint8_t *ppi_src_pointers[2] = { p_src, p_src + i_src_stride * i_slice_height };
         size_t pi_src_strides[2] = { i_src_stride, i_src_stride };
-        CopyFromNv12( p_pic, ppi_src_pointers, pi_src_strides, i_slice_height, p_surface_cache );
+        CopyFromNv12ToYv12( p_pic, ppi_src_pointers, pi_src_strides,
+                            i_slice_height, p_surface_cache );
         return;
     }
 #endif
