@@ -2,7 +2,7 @@
  * ts.c: Transport Stream input module for VLC.
  *****************************************************************************
  * Copyright (C) 2004-2016 VLC authors and VideoLAN
- * $Id: 7fea982ec84f2806fb1b5277152932242b3bef4e $
+ * $Id: 1b100e6dc050a13f77be631ff81ff9ed5d5674ec $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Jean-Paul Saman <jpsaman #_at_# m2x.nl>
@@ -208,7 +208,7 @@ static int DetectPacketSize( demux_t *p_demux, unsigned *pi_header_size, int i_o
         int i_peek = i_offset + TS_PACKET_SIZE_MAX * 3 + i_sync + 1;
         if( ( vlc_stream_Peek( p_demux->s, &p_peek, i_peek ) ) < i_peek )
         {
-            msg_Err( p_demux, "cannot peek" );
+            msg_Dbg( p_demux, "cannot peek" );
             return -1;
         }
         if( p_peek[i_offset + i_sync + 1 * TS_PACKET_SIZE_188] == 0x47 &&
@@ -686,6 +686,7 @@ static int Demux( demux_t *p_demux )
             {
                 msg_Dbg( p_demux, "Creating delayed ES" );
                 AddAndCreateES( p_demux, p_pid, true );
+                UpdatePESFilters( p_demux, p_sys->b_es_all );
             }
 
             /* Emulate HW filter */
@@ -2780,6 +2781,4 @@ void AddAndCreateES( demux_t *p_demux, ts_pid_t *pid, bool b_create_delayed )
                 DoCreateES( p_demux, p_pmt->e_streams.p_elems[j]->u.p_pes->p_es, NULL );
         }
     }
-
-    UpdatePESFilters( p_demux, p_sys->b_es_all );
 }
