@@ -2,7 +2,7 @@
  * interface_widgets.cpp : Custom widgets for the main interface
  ****************************************************************************
  * Copyright (C) 2006-2010 the VideoLAN team
- * $Id: 0f3dd29848265f80ed5fa814d105dd7025430d7d $
+ * $Id: 7ca12a1efeaaf8f52b7dd7ff8a8ca4c4242eb344 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -262,7 +262,8 @@ void VideoWidget::setSize( unsigned int w, unsigned int h )
 
 bool VideoWidget::nativeEvent( const QByteArray& eventType, void* message, long* )
 {
-#if defined(QT5_HAS_XCB)
+#if defined(QT5_HAS_X11)
+# if defined(QT5_HAS_XCB)
     if ( eventType == "xcb_generic_event_t" )
     {
         const xcb_generic_event_t* xev = static_cast<const xcb_generic_event_t*>( message );
@@ -270,6 +271,7 @@ bool VideoWidget::nativeEvent( const QByteArray& eventType, void* message, long*
         if ( xev->response_type == XCB_CONFIGURE_NOTIFY )
             reportSize();
     }
+# endif
 #endif
 #ifdef _WIN32
     if ( eventType == "windows_generic_MSG" )
