@@ -2,7 +2,7 @@
  * h264.c: h264/avc video packetizer
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2006 VLC authors and VideoLAN
- * $Id: 8a91adbf72f5a5f257edcf1650a08e8b74d890ba $
+ * $Id: 989eac8f5003fb7f8354e80a1bd7664fe14d858d $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -135,7 +135,7 @@ struct decoder_sys_t
 
 static block_t *Packetize( decoder_t *, block_t ** );
 static block_t *PacketizeAVC1( decoder_t *, block_t ** );
-static block_t *GetCc( decoder_t *p_dec, bool pb_present[4] );
+static block_t *GetCc( decoder_t *p_dec, bool pb_present[4], int * );
 static void PacketizeFlush( decoder_t * );
 
 static void PacketizeReset( void *p_private, bool b_broken );
@@ -490,8 +490,9 @@ static block_t *PacketizeAVC1( decoder_t *p_dec, block_t **pp_block )
 /*****************************************************************************
  * GetCc:
  *****************************************************************************/
-static block_t *GetCc( decoder_t *p_dec, bool pb_present[4] )
+static block_t *GetCc( decoder_t *p_dec, bool pb_present[4], int *pi_reorder_depth )
 {
+    *pi_reorder_depth = 0;
     return cc_storage_get_current( p_dec->p_sys->p_ccs, pb_present );
 }
 
@@ -838,7 +839,8 @@ static block_t *OutputPicture( decoder_t *p_dec )
 
     if( p_pic->i_pts == VLC_TS_INVALID )
     {
-        if( p_sys->prevdatedpoc.pts > VLC_TS_INVALID )
+        if( p_sys->prevdatedpoc.pts > VLC_TS_INVALID &&
+            date_Get( &p_sys->dts ) != VLC_TS_INVALID )
         {
             date_t pts = p_sys->dts;
             date_Set( &pts, p_sys->prevdatedpoc.pts );
@@ -857,7 +859,8 @@ static block_t *OutputPicture( decoder_t *p_dec )
         {
             p_pic->i_pts = p_pic->i_dts;
         }
-        else if( p_sys->slice.type == H264_SLICE_TYPE_I )
+        else if( p_sys->slice.type == H264_SLICE_TYPE_I &&
+                 date_Get( &p_sys->dts ) != VLC_TS_INVALID )
         {
             /* Hell no PTS on IDR. We're totally blind */
             date_t pts = p_sys->dts;
@@ -898,7 +901,8 @@ static block_t *OutputPicture( decoder_t *p_dec )
     if( !p_sys->b_discontinuity )
     {
         /* save for next pic fixups */
-        date_Increment( &p_sys->dts, i_num_clock_ts );
+        if( date_Get( &p_sys->dts ) != VLC_TS_INVALID )
+            date_Increment( &p_sys->dts, i_num_clock_ts );
     }
     else
     {
